@@ -686,6 +686,8 @@ static void nfs_show_mount_options(struct seq_file *m, struct nfs_server *nfss,
                 seq_puts(m, ",noidlexprt");
 	if (clp->cl_mdconnect)
                 seq_printf(m, ",mdconnect=%d", clp->cl_mdconnect);
+	if (clp->cl_pconnect >= 2)
+                seq_printf(m, ",pconnect=%d", clp->cl_pconnect);
 	if (clp->cl_spread_reads)
 		seq_puts(m, ",spread_reads");
 	if (clp->cl_spread_writes)
