@@ -1,4 +1,3 @@
-// SPDX-License-Identifier: GPL-2.0
 /*
  * Device operations for the pnfs nfs4 file layout driver.
  *
@@ -44,7 +43,6 @@ void nfs4_ff_layout_free_deviceid(struct nfs4_ff_layout_ds *mirror_ds)
 {
 	nfs4_print_deviceid(&mirror_ds->id_node.deviceid);
 	nfs4_pnfs_ds_put(mirror_ds->ds);
-	kfree(mirror_ds->ds_versions);
 	kfree_rcu(mirror_ds, id_node.rcu);
 }
 
@@ -132,13 +130,7 @@ nfs4_ff_alloc_deviceid_node(struct nfs_server *server, struct pnfs_device *pdev,
 		if (ds_versions[i].wsize > NFS_MAX_FILE_IO_SIZE)
 			ds_versions[i].wsize = NFS_MAX_FILE_IO_SIZE;
 
-		/*
-		 * check for valid major/minor combination.
-		 * currently we support dataserver which talk:
-		 *   v3, v4.0, v4.1, v4.2
-		 */
-		if (!((ds_versions[i].version == 3 && ds_versions[i].minor_version == 0) ||
-			(ds_versions[i].version == 4 && ds_versions[i].minor_version < 3))) {
+		if (ds_versions[i].version != 3 || ds_versions[i].minor_version != 0) {
 			dprintk("%s: [%d] unsupported ds version %d-%d\n", __func__,
 				i, ds_versions[i].version,
 				ds_versions[i].minor_version);
@@ -196,7 +188,7 @@ out_err:
 static void ff_layout_mark_devid_invalid(struct pnfs_layout_segment *lseg,
 		struct nfs4_deviceid_node *devid)
 {
-	nfs4_delete_deviceid(devid->ld, devid->nfs_client, &devid->deviceid);
+	nfs4_mark_deviceid_unavailable(devid);
 	if (!ff_layout_has_available_ds(lseg))
 		pnfs_error_mark_layout_for_return(lseg->pls_layout->plh_inode,
 				lseg);
@@ -227,10 +219,6 @@ static bool ff_layout_mirror_valid(struct pnfs_layout_segment *lseg,
 		} else
 			goto outerr;
 	}
-
-	if (IS_ERR(mirror->mirror_ds))
-		goto outerr;
-
 	if (mirror->mirror_ds->ds == NULL) {
 		struct nfs4_deviceid_node *devid;
 		devid = &mirror->mirror_ds->id_node;
@@ -299,9 +287,8 @@ ff_layout_add_ds_error_locked(struct nfs4_flexfile_layout *flo,
 		}
 		/* Entries match, so merge "err" into "dserr" */
 		extend_ds_error(dserr, err->offset, err->length);
-		list_replace(&err->list, &dserr->list);
+		list_del(&err->list);
 		kfree(err);
-		return;
 	}
 
 	list_add_tail(&dserr->list, head);
@@ -431,10 +418,11 @@ nfs4_ff_layout_prepare_ds(struct pnfs_layout_segment *lseg, u32 ds_idx,
 	status = nfs4_pnfs_ds_connect(s, ds, devid, dataserver_timeo,
 			     dataserver_retrans,
 			     mirror->mirror_ds->ds_versions[0].version,
-			     mirror->mirror_ds->ds_versions[0].minor_version);
+			     mirror->mirror_ds->ds_versions[0].minor_version,
+			     RPC_AUTH_UNIX);
 
 	/* connect success, check rsize/wsize limit */
-	if (!status) {
+	if (ds->ds_clp) {
 		max_payload =
 			nfs_block_size(rpc_max_payload(ds->ds_clp->cl_rpcclient),
 				       NULL);
@@ -444,11 +432,11 @@ nfs4_ff_layout_prepare_ds(struct pnfs_layout_segment *lseg, u32 ds_idx,
 			mirror->mirror_ds->ds_versions[0].wsize = max_payload;
 		goto out;
 	}
-out_fail:
 	ff_layout_track_ds_error(FF_LAYOUT_FROM_HDR(lseg->pls_layout),
 				 mirror, lseg->pls_range.offset,
 				 lseg->pls_range.length, NFS4ERR_NXIO,
 				 OP_ILLEGAL, GFP_NOIO);
+out_fail:
 	if (fail_return || !ff_layout_has_available_ds(lseg))
 		pnfs_error_mark_layout_for_return(ino, lseg);
 	ds = NULL;
@@ -494,26 +482,20 @@ nfs4_ff_find_or_create_ds_client(struct pnfs_layout_segment *lseg, u32 ds_idx,
 	}
 }
 
-void ff_layout_free_ds_ioerr(struct list_head *head)
-{
-	struct nfs4_ff_layout_ds_err *err;
-
-	while (!list_empty(head)) {
-		err = list_first_entry(head,
-				struct nfs4_ff_layout_ds_err,
-				list);
-		list_del(&err->list);
-		kfree(err);
-	}
-}
-
 /* called with inode i_lock held */
-int ff_layout_encode_ds_ioerr(struct xdr_stream *xdr, const struct list_head *head)
+int ff_layout_encode_ds_ioerr(struct nfs4_flexfile_layout *flo,
+			      struct xdr_stream *xdr, int *count,
+			      const struct pnfs_layout_range *range)
 {
-	struct nfs4_ff_layout_ds_err *err;
+	struct nfs4_ff_layout_ds_err *err, *n;
 	__be32 *p;
 
-	list_for_each_entry(err, head, list) {
+	list_for_each_entry_safe(err, n, &flo->error_list, list) {
+		if (!pnfs_is_range_intersecting(err->offset,
+				pnfs_end_offset(err->offset, err->length),
+				range->offset,
+				pnfs_end_offset(range->offset, range->length)))
+			continue;
 		/* offset(8) + length(8) + stateid(NFS4_STATEID_SIZE)
 		 * + array length + deviceid(NFS4_DEVICEID4_SIZE)
 		 * + status(4) + opnum(4)
@@ -532,59 +514,17 @@ int ff_layout_encode_ds_ioerr(struct xdr_stream *xdr, const struct list_head *he
 					    NFS4_DEVICEID4_SIZE);
 		*p++ = cpu_to_be32(err->status);
 		*p++ = cpu_to_be32(err->opnum);
-		dprintk("%s: offset %llu length %llu status %d op %d\n",
+		*count += 1;
+		list_del(&err->list);
+		dprintk("%s: offset %llu length %llu status %d op %d count %d\n",
 			__func__, err->offset, err->length, err->status,
-			err->opnum);
+			err->opnum, *count);
+		kfree(err);
 	}
 
 	return 0;
 }
 
-static
-unsigned int do_layout_fetch_ds_ioerr(struct pnfs_layout_hdr *lo,
-				      const struct pnfs_layout_range *range,
-				      struct list_head *head,
-				      unsigned int maxnum)
-{
-	struct nfs4_flexfile_layout *flo = FF_LAYOUT_FROM_HDR(lo);
-	struct inode *inode = lo->plh_inode;
-	struct nfs4_ff_layout_ds_err *err, *n;
-	unsigned int ret = 0;
-
-	spin_lock(&inode->i_lock);
-	list_for_each_entry_safe(err, n, &flo->error_list, list) {
-		if (!pnfs_is_range_intersecting(err->offset,
-				pnfs_end_offset(err->offset, err->length),
-				range->offset,
-				pnfs_end_offset(range->offset, range->length)))
-			continue;
-		if (!maxnum)
-			break;
-		list_move(&err->list, head);
-		maxnum--;
-		ret++;
-	}
-	spin_unlock(&inode->i_lock);
-	return ret;
-}
-
-unsigned int ff_layout_fetch_ds_ioerr(struct pnfs_layout_hdr *lo,
-				      const struct pnfs_layout_range *range,
-				      struct list_head *head,
-				      unsigned int maxnum)
-{
-	unsigned int ret;
-
-	ret = do_layout_fetch_ds_ioerr(lo, range, head, maxnum);
-	/* If we're over the max, discard all remaining entries */
-	if (ret == maxnum) {
-		LIST_HEAD(discard);
-		do_layout_fetch_ds_ioerr(lo, range, &discard, -1);
-		ff_layout_free_ds_ioerr(&discard);
-	}
-	return ret;
-}
-
 static bool ff_read_layout_has_available_ds(struct pnfs_layout_segment *lseg)
 {
 	struct nfs4_ff_layout_mirror *mirror;
