@@ -52,7 +52,7 @@
 #include <linux/sunrpc/addr.h>
 #include <linux/sunrpc/xprt.h>
 #include <linux/rcupdate.h>
-#include <linux/sched/mm.h>
+#include <linux/sched.h>
 
 #include <trace/events/sunrpc.h>
 
@@ -633,7 +633,7 @@ EXPORT_SYMBOL_GPL(xprt_write_space);
 
 static unsigned long xprt_abs_ktime_to_jiffies(ktime_t abstime)
 {
-	s64 delta = ktime_to_ns(ktime_get() - abstime);
+	s64 delta = ktime_to_ns(ktime_sub(ktime_get(), abstime));
 	return likely(delta >= 0) ?
 		jiffies - nsecs_to_jiffies(delta) :
 		jiffies + nsecs_to_jiffies(-delta);
@@ -712,14 +712,14 @@ static void xprt_autoclose(struct work_struct *work)
 {
 	struct rpc_xprt *xprt =
 		container_of(work, struct rpc_xprt, task_cleanup);
-	unsigned int pflags = memalloc_nofs_save();
+	unsigned int pflags = memalloc_noio_save();
 
 	trace_xprt_disconnect_auto(xprt);
 	clear_bit(XPRT_CLOSE_WAIT, &xprt->state);
 	xprt->ops->close(xprt);
 	xprt_release_write(xprt, NULL);
 	wake_up_bit(&xprt->state, XPRT_LOCKED);
-	memalloc_nofs_restore(pflags);
+	memalloc_noio_restore(pflags);
 }
 
 /**
@@ -1814,8 +1814,8 @@ xprt_request_init(struct rpc_task *task)
 	req->rq_snd_buf.buflen = 0;
 	req->rq_rcv_buf.len = 0;
 	req->rq_rcv_buf.buflen = 0;
-	req->rq_snd_buf.bvec = NULL;
-	req->rq_rcv_buf.bvec = NULL;
+	req->rq_snd_buf.iovec = NULL;
+	req->rq_rcv_buf.iovec = NULL;
 	req->rq_release_snd_buf = NULL;
 	xprt_init_majortimeo(task, req);
 	dprintk("RPC: %5u reserved req %p xid %08x\n", task->tk_pid,
