@@ -510,7 +510,7 @@ static int rsc_parse(struct cache_detail *cd,
 			kgid = make_kgid(current_user_ns(), id);
 			if (!gid_valid(kgid))
 				goto out;
-			GROUP_AT(rsci.cred.cr_group_info, i) = kgid;
+			rsci.cred.cr_group_info->gid[i] = kgid;
 		}
 		groups_sort(rsci.cred.cr_group_info);
 
@@ -1251,7 +1251,7 @@ static int gss_proxy_save_rsc(struct cache_detail *cd,
 		 * mapping to nobody */
 		goto out;
 	} else {
-		struct timespec64 boot;
+		struct timespec boot;
 
 		/* steal creds */
 		rsci.cred = ud->creds;
@@ -1273,7 +1273,7 @@ static int gss_proxy_save_rsc(struct cache_detail *cd,
 		if (status)
 			goto out;
 
-		getboottime64(&boot);
+		getboottime(&boot);
 		expiry -= boot.tv_sec;
 	}
 
