@@ -5,8 +5,6 @@
  * Use the core R/W API to move RPC-over-RDMA Read and Write chunks.
  */
 
-#include "overflow.h"
-
 #include <rdma/rw.h>
 
 #include <linux/sunrpc/rpc_rdma.h>
@@ -73,12 +71,9 @@ svc_rdma_get_rw_ctxt(struct svcxprt_rdma *rdma, unsigned int sges)
 
 	ctxt->rw_sg_table.sgl = ctxt->rw_first_sgl;
 	if (sg_alloc_table_chained(&ctxt->rw_sg_table, sges,
-				   ctxt->rw_sg_table.sgl)) {
-		kfree(ctxt);
-		ctxt = NULL;
+				   ctxt->rw_sg_table.sgl,
+				   SG_CHUNK_SIZE))
 		goto out_free;
-	}
-
 	return ctxt;
 
 out_free:
@@ -91,7 +86,7 @@ out_noctx:
 static void svc_rdma_put_rw_ctxt(struct svcxprt_rdma *rdma,
 				 struct svc_rdma_rw_ctxt *ctxt)
 {
-	sg_free_table_chained(&ctxt->rw_sg_table, true);
+	sg_free_table_chained(&ctxt->rw_sg_table, SG_CHUNK_SIZE);
 
 	spin_lock(&rdma->sc_rw_ctxt_lock);
 	list_add(&ctxt->rw_list, &rdma->sc_rw_ctxts);
